﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Infragistics.Win;
using Infragistics.Win.UltraWinEditors;

namespace WG_DifferentInputMasksSameColumn
{
    public class EditorCache : IDisposable
    {


        private Dictionary<string, EmbeddableEditorBase> editorsDictionary;
        public EmbeddableEditorBase GetEditor(string inputMask)
        {
            // Try to get an existing, cached editor component. 
            EmbeddableEditorBase editor = this.GetExistingEditor(inputMask);
            if (editor != null)
            {
                return editor;
            }

            // If we failed to find an existing editor component, create a new one and 
            // cache it. 
            return this.CreateNewEditor(inputMask);
        }

        // Try to find an exiting editor component in the cache. 
        private EmbeddableEditorBase GetExistingEditor(string inputMask)
        {
            // If the editorsDictionary is null, nothing is cached, so 
            // just return null. 
            if (this.editorsDictionary == null)
            {
                return null;
            }

            // Try to find an editor component with the specified mask. 
            EmbeddableEditorBase editor = default(EmbeddableEditorBase);
            bool success = this.editorsDictionary.TryGetValue(inputMask, out editor);

            // If we found an existing editor component, return it, otherwise, return null.
            return success ? editor : null;
        }

        private EmbeddableEditorBase CreateNewEditor(string inputMask)
        {
            DefaultEditorOwnerSettings settings = new DefaultEditorOwnerSettings();
            settings.MaskInput = inputMask;

            DefaultEditorOwner owner = new DefaultEditorOwner(settings);
            // Create a new editor component with the appropriate settings. 
            EditorWithMask editor = new EditorWithMask(owner);

            // Cache the editor component
            if (this.editorsDictionary == null)
            {
                this.editorsDictionary = new Dictionary<string, EmbeddableEditorBase>();
            }

            this.editorsDictionary.Add(inputMask, editor);

            // return the new editor component. 
            return editor;
        }

        #region "IDisposable Members"

        public void Dispose()
        {
            if (this.editorsDictionary != null)
            {
                EmbeddableEditorBase[] editors = new EmbeddableEditorBase[this.editorsDictionary.Count];
                this.editorsDictionary.Values.CopyTo(editors, 0);
                foreach (EmbeddableEditorBase editor in editors)
                {
                    ((IDisposable)editor).Dispose();
                }
            }
        }

        #endregion

    }

}
